#include "edecfunc.h"
#include <sys/types.h>

#define MaxOf(a, b)  (a > b) ? a : b
#define MinOf(a, b)  (a < b) ? a : b

void setup_colors(void),
     show_all_colors(void);

/*******************  put an image ***********/
void getimage(short x, short y, short x1, short y1, 
                                      Pixmap *image){
    /*First, create the pixmap*/
    *image = XCreatePixmap(D,W,x1-x+1,y1-y+1,screen_depth);
	
    if (*image == 0) {
       fprintf(stderr, "%s\n", "Error creating pixmap");
       return;
    }	
    XCopyArea(D,W,*image,Gc,x,y,x1-x+1,y1-y+1,0,0);
}


/*******************  put an image ************/   
void putimage(short x, short y, Pixmap *image, 
                  short drawing_mode){
 Window	root_window;
 int	top, left;
 unsigned int	border, depth;
 unsigned int	width, height;
	
    if (*image == 0) {
       fprintf(stderr,
              "%s\n", "Zero image passed to putimage");
       return;
    }				
    if (XGetGeometry(D,*image, &root_window, 
      &top, &left, &width, &height, &border, &depth) == 0)
		fprintf(stderr, "%s\n", 
            "Failed to get Pixmap geometry in putimage");
    XCopyArea(D,*image,W,Gc,0,0,width,height,x,y);
}

/******************* put_partial_image ************/
void copy_part_image(Drawable *source_d, Drawable *dest_d,
			short source_x, short source_y,
			short dest_x, short dest_y,
			short width, short height) {

    if (*source_d == 0) {
       fprintf(stderr, "%s \n", "Zero source image in copy_part_image");
       return;
    }		
    if (*dest_d == 0){
         fprintf(stdout,"%s\n","Null pixmap passed to copy_part_image");
	 *dest_d = XCreatePixmap(D,W,width, height,screen_depth);
    }

    XCopyArea(D,*source_d,*dest_d,Gc,
            source_x,source_y,width,height,dest_x,dest_y);
}

/*******************  free an image ************/
void freeimage(Pixmap *image){
	if (*image == 0) {
	   fprintf(stderr, "%s\n", "Zero image passed to freeimage");
	   return;
	}
	XFreePixmap(D,*image);
	*image = 0;
}

/*******************  delay for less that one second ************/     
void subsec_delay(long microsec){
  long starttime_m;
  struct timeval tp;
  struct timezone tzp;                   
      
      sprintf(textb[0],"tp");
      sprintf(textb[0],"tzp");

      gettimeofday(&tp,&tzp);
      starttime_m=tp.tv_usec;
      
      while (1){
         gettimeofday(&tp,&tzp);         
         if (starttime_m<tp.tv_usec){  
            if (tp.tv_usec-starttime_m>=microsec) break;
         }
         else if (starttime_m>tp.tv_usec){  
            if ((1000000-starttime_m)+tp.tv_usec>=microsec) break;
         }
      }

}
/**************  print text ****************/
void print_out(char *array){
    /* dummy */
}
/****************** turn vga x coord to general coord *********/ 
signed short vgax(signed short xcoord){ 
   return((signed short) (xcoord *((float)(Maxx+1)/640))); 
} 
/********* turn vga x coord to general coord*************/ 
signed short vgay(signed short ycoord){ 
   return((signed short) (ycoord *((float)(WinMaxy+1)/480))); 
} 
/********* move the ref coord to new poss ************/
void moveto(signed short xpos, signed short ypos){
   getx=xpos;    
   gety=ypos;
}
/********* move the ref coord a given distance ************/
void moverel(signed short xpos, signed short ypos){
   getx+=xpos;    
   gety+=ypos;
}
/*********** get width of text string *****************/
signed short textwidth(char *textbuf){
     return(strlen(textbuf)*CWidth);        
}
/*********** get hieght of text character**************/
signed short textheight(char *textbuf){
     return(CWidth);        
}
/*********** write text and update ref coords **************/
void outtext(char *textbuf){
   moverel(0,CHeight-2);
   XDrawString(D,W,Gc,getx,gety,textbuf,strlen(textbuf));
   /*moverel((strlen(textbuf))*CWidth,-CHeight+2);*/
   if (fonttype==0)
     moverel(XTextWidth(font,textbuf,strlen(textbuf)),-CHeight+2);
   else  moverel(XTextWidth(propfont,textbuf,strlen(textbuf)),-CHeight+2);
   
}

/*********** set the current foreground color **************/
void setcolor(unsigned char thiscolor){
   XSetForeground(D,Gc,Color_Table[(thiscolor)]);
   nowcolor=thiscolor;
}
/*********** get the current foreground color **************/
unsigned char getcolor(void){
   return(nowcolor);
}
		   
/*********** draw single pixel in given color  **************/
void putpixel(signed short xpos, signed short ypos,unsigned char color){
	unsigned char tempcolor;
	
	tempcolor=nowcolor;
	setcolor(color);
	XDrawPoint(D,*current_draw, Gc,xpos,ypos);
	setcolor(tempcolor);         
}


/*********** clip graphics in a rectangle **************/
void setviewport(signed short left,signed short top, 
    signed short right, signed short bottom, 
                 signed short clip){        
   viewport[0].x=left;
   viewport[0].y=top;
   viewport[0].width=right-left+1;
   viewport[0].height=bottom-top+1;
   moveto(left,top);
   XSetClipRectangles(D,Gc,0,0,viewport,1,Unsorted);
}

/**** erase viewport, ie fill with background color *****/
void clearviewport(void){
 unsigned char tempcolor; 
   tempcolor=nowcolor;
   setcolor(black);  /*background color*/
   /*draw a filled rectangle as large as the current viewport*/
    XFillRectangle(D,*current_draw,Gc,viewport[0].x,viewport[0].y,
           viewport[0].x+viewport[0].width,viewport[0].y+viewport[0].height);
   setcolor(tempcolor);
}

/* draw line given distance  from ref coords to new coords, update *****/
void linerel(signed short dx,signed short dy){
   line(getx,gety,getx+dx,gety+dy);
   moveto(getx+dx,gety+dy);
} 
/**** draw line from ref coords to new coords, update *****/
void lineto(signed short x,signed short y){
    line(getx,gety,x,y);
    moveto(x,y);
}
/*********** set the fill color and pattern  **************/
void setfillstyle(signed short pattern, signed short color){
   fillcolor=color;
   XSetFillStyle(D,Gc,pattern);
}
/********* draw rectangular colored block, no outline ******/
void bar(signed short a,signed short b,
         signed short a1,signed short b1){
 unsigned char tempcolor;
   tempcolor=nowcolor;
   setcolor(fillcolor);
   XFillRectangle(D,*current_draw,Gc,a,b,a1-a+1,b1-b+1);
   setcolor(tempcolor);
}

/***************** draw filled rectangle *******************/

void bar3d(signed short a,signed short b,
    signed short a1,signed short b1,
      signed short dummy1,signed short dummy2){
  unsigned char tempcolor;
   tempcolor=nowcolor;
   setcolor(fillcolor);
   
   XFillRectangle(D,*current_draw,Gc,a,b,a1-a+1,b1-b+1);
   setcolor(tempcolor);
   XDrawRectangle(D,*current_draw,Gc,a,b,a1-a,b1-b);
}

/****************** draw filled poly **********************/
void fillpoly(signed short numpoints,int polypoints[]){
  unsigned char tempcolor;
  XPoint points[numpoints];
  unsigned short z;
   
   for (z=0;z<=numpoints-1;z++){
        points[z].x=polypoints[z*2];
        points[z].y=polypoints[z*2+1];
   }
   tempcolor=nowcolor;
   setcolor(fillcolor);
   XFillPolygon(D,W,Gc,points,numpoints,Convex,CoordModeOrigin);
   setcolor(tempcolor);
}
/******** draw circle and fill with given color *************/
void pieslice(signed short x,signed short y, 
      signed short stangle,signed short entangle,signed short radius){
  unsigned char tempcolor;
   tempcolor=nowcolor;
   setcolor(fillcolor);
   XFillArc(D,W,Gc,x-radius,y-radius,radius*2,radius*2,0,360*64);
   setcolor(tempcolor);
   circle(x,y,radius);
}
/******************** window width  *************************/
signed short getmaxx(void){
   return(Maxx);
}
/******************** window height  *************************/
signed short getmaxy(void){
   return(WinMaxy);
}
/********************* get the color of a pixel **************/
/*this is really nasty!*/
unsigned char getpixel(signed short x,signed short y){ 
	XImage *onepixel;
	   
    onepixel = XGetImage(D,W,x,y,1,1,AllPlanes,XYPixmap);
    
    if (onepixel != NULL)
    	return ((unsigned char)(XGetPixel(onepixel,0,0)));
    else
    	return 0;
}
/*********************** print the window  ********************/
void printimage(void){
  char 		 cmd[1024];
  unsigned short xbox,boxx,boxy,tempx,tempy;
  Pixmap	 *printbox;
  
    tempx=getx;
    tempy=gety;
    if ((printbox = (Pixmap *)malloc(sizeof(Pixmap *))) == NULL)
		outof_memory("printbox");	
    *printbox=0;

    boxx=Maxx/2;
    boxy=2;
    getimage(boxx,boxy,Maxx-1,boxy+CHeight*2+vgay(6),printbox);
    xbox=draw_prompt_box(2);
    outtext(
       "print: without frame -p, with frame -P, for no greyscale add -n");
    moveto(xbox,gety+CHeight);
    outtext(
       "then click mouse button in window, quit-q:");

    input_string(2,0);
    if (is[0]=='p'|| is[0]=='P') putimage(boxx,boxy,printbox,COPY_PUT);
    else{
	is[0]=0;
	putimage(boxx,boxy,printbox,COPY_PUT);		
        freeimage(printbox);
        free(printbox);
	return;
    }
    XFlush(D);
    if (is[0]=='p'){
       XFlush(D);
       sprintf(cmd,"xwd > temp_window_file.im"); /*without frame*/
    }
    else{ 
        XFlush(D);
        sprintf(cmd,"xwd -frame > temp_window_file.im"); /*with frame*/    
    }
    system(cmd);
    XFlush(D);

    if (is[1]=='n')  /*without grey scale*/
       sprintf(cmd,
"xpr -device ps -compact -cutoff 99 temp_window_file.im > temp_window_file.ps");
    else  /*with grey scale*/    
       sprintf(cmd,
"xwdtopnm temp_window_file.im | ppmtopgm | pnmtops > temp_window_file.ps");
    system(cmd);
    
    draw_prompt_box(1);
    outtext(" wait");
    putimage(boxx,boxy,printbox,COPY_PUT);
    sprintf(cmd,"lpr temp_window_file.ps");
    system(cmd);
        
    draw_prompt_box(1);
    outtext(" done - press any key to continue");
    input_string(1,0);
    putimage(boxx,boxy,printbox,COPY_PUT);
    freeimage(printbox);
    free(printbox);
    moveto(tempx,tempy);
}
/************************* length of file  *************************/
unsigned long filelength(signed short fid){
    struct stat buf;

    fstat(fid,&buf); 
    return(buf.st_size);
}

/************************* graphics setup  *************************/
void open_graphics(char firstgraphics){    
	char *display_name = NULL;
	unsigned long		changed_values;
	Window			parent;
	XSetWindowAttributes	setWindowAttr;
	XWindowAttributes	windowAttr;
	XGCValues               gcValues;
        
    if (firstgraphics==0){
       /*superVGA=1024  VGA=640*/
       /*        =768      =480*/       
       /*
       if (XDisplayWidth(D,S)>=1152 && XDisplayHeight(D,S)>=900){
          Maxx =1023;   
          WinMaxy =767;  
       }
       */
       /*for smaller screen*/       
       /*
       else{
          Maxx = XDisplayWidth(D,S)-100; 
          WinMaxy = Maxx*480/640; 
       }*/

       if (bground==0) bground='w';  /*background color default white*/
       Maxx =1023;   	     /*superVGA 1024  639*/
       WinMaxy =767;         /*768  479*/
              
       Maxy=WinMaxy-vgay(16);/*window less bottom title bar*/
    }

    if ((D = XOpenDisplay(display_name)) == NULL){
    	fprintf(stderr, "%s: cannot connect to X server %s\n",
    				"DDLab", XDisplayName(display_name));
    	exit(-1);
    }
    
    S = DefaultScreen (D);
    
    if (XDisplayWidth(D,S)-100 < Maxx)
                      Maxx = XDisplayWidth(D,S) - 100; 
    WinMaxy = Maxx * 480/640;
    Maxy = WinMaxy - vgay(16);
	
    /*Prepare window variables*/
    parent =  DefaultRootWindow (D);	
	
    /*Create a simple window*/
    if (bground=='w'){ /*white background*/
       W = XCreateSimpleWindow (D, parent,
	      0, 0,		 /*x, y*/
	      Maxx+1, WinMaxy+1, /*height, width*/
	      1,		 /*border width*/
	      BlackPixel(D, S),  /*border color*/
	      WhitePixel(D, S)); /*window background color*/
    }
    else if (bground=='b'){  /*black background*/   
       W = XCreateSimpleWindow (D, parent,
	      0, 0,				/*x, y*/
	      Maxx+1, WinMaxy+1, /*height, width*/
	      1,		 /*border width*/
	      WhitePixel(D, S),  /*border color*/
	      BlackPixel(D, S)); /*window background color*/
    }
    /*does this server support window backing stores?*/	
    if (XDoesBackingStore(&D->screens[S])){
    	fprintf(stderr,"%s\n", "Using backing store");
		changed_values = CWBackingStore;
		
		/*Now change the window attributes*/
		setWindowAttr.backing_store = WhenMapped;
	
		XChangeWindowAttributes(D, W, changed_values, &setWindowAttr);
    }
    
    /*get the screen depth (used by get image)*/
    XGetWindowAttributes(D, W, &windowAttr);
    screen_depth = windowAttr.depth;
        
    XSelectInput(D, W, ExposureMask|KeyPressMask|ButtonPressMask
             |ButtonReleaseMask|PointerMotionMask|StructureNotifyMask);
   
    XStoreName(D, W, "Discrete Dynamics Lab");
    XMapRaised(D, W);

    gcValues.graphics_exposures = 0;

    Gc = XCreateGC (D, W, GCGraphicsExposures, &gcValues);

    XSetForeground(D, Gc, BlackPixel(D, S));
    XSetBackground(D, Gc, WhitePixel(D, S));
    XSetLineAttributes(D, Gc, 0, LineSolid, CapButt, JoinMiter);
    
    /*Wait for the first expose event. Then we know that the window is visible*/
    do {
    	XNextEvent(D, &Event);
    } while (Event.type != Expose);
    
    leftCursor =XCreateFontCursor(D,XC_left_ptr);
    rightCursor =XCreateFontCursor(D,XC_right_ptr);
    XDefineCursor(D,W,rightCursor);
    mouse_found=1;

    /*if (firstgraphics==0) pointsize=14;
    else */
    setup_fonts();
    setup_colors();
    if (firstgraphics==0) setup_images();
        
    *current_draw = W;
}



/***************** setup_images ************************/
void setup_images(void){
	short	i;
	
	for (i=0; i < kNumBoxImages; i++) {
	   boximage[i] = 
                (Pixmap *)malloc(sizeof(Pixmap));
	   *boximage[i] = 0;
	}
	
	for (i=0; i < kNumBoxPromptImages; i++) {
	   boximage_prompt[i] = 
                 (Pixmap *)malloc(sizeof(Pixmap));
	   *boximage_prompt[i] = 0;
	}
	
	stpimage = (Pixmap *)malloc(sizeof(Pixmap));
	*stpimage = 0;
	
	statblock = (Pixmap *)malloc(sizeof(Pixmap));
	*statblock = 0;
	
	current_draw = (Drawable *)malloc(sizeof(Drawable));
	*current_draw = 0;
       
}
/**************** free_images *************/
void free_images(void){
   short	i;
	
	for (i=0; i < kNumBoxImages; i++) {
		if (*boximage[i] != 0)
			freeimage(boximage[i]);
	}
	for (i=0; i < kNumBoxPromptImages; i++){
	   if (*boximage_prompt[i] != 0)
		freeimage(boximage_prompt[i]);
	}	
	if (*stpimage != 0) freeimage(stpimage);
	if (*statblock != 0) freeimage(statblock);        
}

/******** close the graphics, and free up stuff ******/
void close_graphics(void)
{

	free_images();
	
	if (font != NULL)
		XUnloadFont(D, font->fid);		
	if (propfont != NULL)
		XUnloadFont(D, propfont->fid);
	if (italicfont != NULL)
		XUnloadFont(D, italicfont->fid);
	XFreeCursor(D, leftCursor);
	XFreeCursor(D, rightCursor);
		
	XDestroyWindow(D,W);
	XFreeGC(D,Gc);
	XCloseDisplay(D);

}


/******** reset Maxx and Maxy after the window has been resized ******/
/*Simon 1/1/96*/
void resize_window(void){
   XWindowAttributes  windowAttr;
   signed short oldMaxx,oldWinMaxy,tempx,tempy;
        
        oldMaxx=Maxx;
        oldWinMaxy=WinMaxy;
        tempx=getx;
        tempy=gety;

	if (XGetWindowAttributes(D, W, &windowAttr) == 0){	
	   fprintf(stderr, "%s: Failed to get window attributes.\n",
			"DDLab");
	   exit(-1);
	}
        
	Maxx = windowAttr.width-1;
	WinMaxy = windowAttr.height-1;
	Maxy = WinMaxy - vgay(16);
	
	setup_fonts();	
        if (oldMaxx!=Maxx || oldWinMaxy!=WinMaxy){            
           draw_bottom_title();
           sprintf(textb[0],
               "pointsize=%d screen=%dx%d",pointsize,Maxx+1,WinMaxy+1);        
           draw_hint_box(0,textb[0],385);
           moveto(tempx*(float)Maxx/oldMaxx,tempy*(float)oldWinMaxy/WinMaxy);
        }

}


/*************** Redraw window after it is uncovered ********************/
void redraw_window(void) {
	
	draw_bottom_title();

}

/***************** revise font size ************************/
void setup_fonts(void) {

    pointsize=ceil(14.0 * (float)(Maxx+1)/1024);
	
    if (pointsize%2==1)
    	pointsize--;
    	
    if (pointsize < 10)
    	pointsize=10;
    

    /*load non-prop font*/
    sprintf(textb[0],"%s%d%s",
             "-adobe-courier-bold-r-*-*-0-",pointsize*10,"-*-*-*-0-*-*");
    
    /*"-*-*-*-r-*-*-0-",0,"-*-*-*-0-*-*");*/
    /*"-bitstream-courier-medium-r-normal-*-0-",pointsize*10,"-100-100-m-0-iso8859-1"); */ 
    /*"-*-lucidatypewriter-bold-r-normal-*-0-",pointsize*10,"-*-*-*-*-*-*");*/
     
    font = XLoadQueryFont(D,textb[0]);   
    
    if (font!=NULL){ 
          XSetFont(D,Gc,font->fid);
          CHeight=font->ascent+font->descent;
          CWidth=font->max_bounds.width;
    }
    else {
          printf("selected non-proportional font not found");
          CHeight=11;
          CWidth=6;
    }
    
    /*load proportional font*/       
    sprintf(textb[0],"%s%d%s",
           "-*-times-bold-r-*-*-*-",pointsize*10,"-*-*-*-*-*-*");  
    propfont=XLoadQueryFont(D,textb[0]);
    
    if (propfont!=NULL){ 
       XSetFont(D,Gc,propfont->fid);    
       fonttype=1;
       CHeight=font->ascent+font->descent;
       CWidth=font->max_bounds.width;
    }
    else printf("selected proportional font not found");
    /*load italic proportional font*/       
    sprintf(textb[0],"%s%d%s",
           "-*-times-bold-i-*-*-*-",pointsize*10,"-*-*-*-*-*-*");  
    italicfont=XLoadQueryFont(D,textb[0]);
   
}

/******** revise window size, text font/size ?, backgrund color ******/
void revise_graphics(void){
    signed short xbox,tempMaxx,newMaxx,tempWinMaxy,
                 newheight,minx,miny; 
   
  start: 
    xbox=draw_prompt_box(4);
    setcolor(white);
    sprintf(textb[0],"character width=%d height=%d pointsize=%d",
                                        CWidth,CHeight,pointsize);
    outtext(textb[0]);   
    sprintf(textb[0],
      "window size=%dx%d resize-s",Maxx+1,WinMaxy+1);	 
    moveto(xbox,gety+CHeight);
    outtext(textb[0]);
    moveto(xbox,gety+CHeight);
    outtext("colors: toggle background-b showold-o showall-a");
    moveto(xbox,gety+CHeight);
    outtext("print-p, quit-q, cont-ret:");
    input_string(1,0);
    if (is[0]=='q' || is[0]==13) return;
    if (is[0]=='p'){
      printimage();
      goto start;
    }
    else if (is[0]=='b'){
       if (bground=='w'){ 
          bground='b';
          XSetWindowBackground(D,W,BlackPixel(D, S));
          XSetWindowBorder(D,W, WhitePixel(D, S));
       }
       else if (bground=='b'){ 
          bground='w';
          XSetWindowBackground(D,W, WhitePixel(D, S));
          XSetWindowBorder(D,W, BlackPixel(D, S));
       }
       setup_colors();
       show_old_colors();
       goto start;
       /*XDestroyWindow(D,W); 
       open_graphics(1);*/       
    }
    else if (is[0]=='o'){
       show_old_colors();
       goto start;
    }
    else if (is[0]=='a'){
       show_all_colors();
       goto start;
    }
    else if (is[0]=='s'){
       minx=300;
       miny=250;
       tempMaxx=Maxx;
       tempWinMaxy=WinMaxy;
       xbox=draw_prompt_box(3);
       setcolor(white);
       sprintf(textb[0],"new window size (min %dx%d)",minx,miny);
       outtext(textb[0]);
       moveto(xbox,gety+CHeight);
       sprintf(textb[0],"width (default %d):",Maxx+1);
       outtext(textb[0]);
       input_string(4,0);
       if (atoi(is)>=minx) newMaxx=atoi(is)-1;
       else newMaxx=Maxx;
       moveto(xbox,gety+CHeight);

       newheight=(newMaxx+1)*480/640;
       if (newheight<miny) newheight=miny;
       sprintf(textb[0],"height (default %d):",newheight);
       outtext(textb[0]);
       input_string(4,0);

       Maxx=newMaxx;
       if (atoi(is)>=200) WinMaxy=atoi(is)-1;
       else WinMaxy=((Maxx+1)*480/640)-1;
       Maxy=WinMaxy-vgay(16);/*window less bottom title bar*/
       if (tempMaxx!=Maxx || tempWinMaxy!=WinMaxy){
          XResizeWindow(D,W,Maxx+1,WinMaxy+1);	
	  setup_fonts();	
	  draw_bottom_title();
          show_old_colors();
          goto start;

	  /* XUnmapWindow(D,W);
          XDestroyWindow(D,W);            
          open_graphics(1);*/
       }
    }
}
/************************* setup_colors ********************************/
void setup_colors(void){
  unsigned char i;   
  signed short  Number_of_colors;
  static char * color_name[]= {
        "White","Black","Yellow", "Gold", "Khaki", "Wheat", 
        "Tan", "Goldenrod", "Orange","Coral","Salmon", 
        "Red", "Orange Red", "Indian Red", "Firebrick", 
        "Brown","Sienna", "Maroon", "Medium Violet Red", 
        "Violet Red","Pink", "Green", "Spring Green", 
        "Medium Spring Green", "Green Yellow",
        "Pale Green","Yellow Green", "Lime Green", 
        "Medium Sea Green","Sea Green", "Forest Green",
        "Olive Drab", "Dark Olive Green", "Dark Green", 
        "Cadet Blue","Medium Aquamarine","Dark Turquoise", 
        "Turquoise", "Aquamarine", "Cyan","Blue", 
        "Medium Blue", "Medium Slate Blue", 
        "Cornflower Blue","Sky Blue", "Light Blue", 
        "Light Steel Blue", "Steel Blue","Dark Slate Blue",
        "Navy", "Navy Blue", "Midnight Blue",
        /******** extra **********/
        "GhostWhite"
   };     
    
    nowcolor=             0; /*keeptrack of current forground color*/
    White=                0;
    Black=                1;
    Yellow=               2;
    Gold=                 3; 
    Khaki=                4;
    Wheat=                5;
    Tan=                  6;
    Goldenred=            7;
    Orange=               8;
    Coral=                9;
    Salmon=               10;
    Red=                  11;
    Orange_Red=           12;
    Indian_Red=           13;
    Firebrick=            14;
    Brown=                15;
    Sienna=               16;
    Maroon=               17;
    Medium_Violet_Red=    18;
    Violet_Red=           19;
    Pink=                 20;
    Green=                21;
    Spring_Green=         22;
    Medium_Spring_Green=  23;
    Green_Yellow=         24;
    Pale_Green=           25;
    Yellow_Green=         26;
    Lime_Green=           27;
    Medium_Sea_Green=     28;
    Sea_Green=            29;
    Forest_Green=         30;
    Olive_Drab=           31;
    Dark_Olive_Green=     32;
    Dark_Green=           33;
    Cadet_Blue=           34;
    Medium_Aquamarine=    35;
    Dark_Turquoise=       36;
    Turquoise=            37;
    Aquamarine=           38;
    Cyan=                 39;
    Blue=                 40;
    Medium_Blue=          41;
    Medium_Slate_Blue=    42;
    Cornflower_Blue=      43;
    Sky_Blue=             44;
    Light_Blue=           45;
    Light_Steel_Blue=     46; 
    Steel_Blue=           47;
    Dark_Slate_Blue=      48;
    Navy=                 49;
    Navy_Blue=            50;
    Midnight_Blue=        51;
    /******** extra *****/
    Ghost_White=           52;
    
    /*old color names*/
    if (bground=='w'){ /*default white background*/                
                black=       oldcolor[0]=White;
		blue=        oldcolor[1]=Gold;                 
		green=       oldcolor[2]=Medium_Sea_Green;   
		lightcyan=   oldcolor[3]=Brown;            
		lightred=    oldcolor[4]=Red;               
		magenta=     oldcolor[5]=Violet_Red;      
		brown=       oldcolor[6]=Dark_Green;      
		lightgray=   oldcolor[7]=Dark_Turquoise; 
		darkgray=    oldcolor[8]=Ghost_White;
		lightblue=   oldcolor[9]=Cornflower_Blue;
		lightgreen=  oldcolor[10]=Lime_Green;    
		cyan=        oldcolor[11]=Olive_Drab;           
		red=         oldcolor[12]=Indian_Red;     
		lightmagenta=oldcolor[13]=Violet_Red; 
		yellow=      oldcolor[14]=Blue;
		white=       oldcolor[15]=Black;
    }
    else if (bground=='b'){ /*alternative black background*/    
                black=       oldcolor[15]=Black;  
		blue=        oldcolor[14]=Blue;
		green=       oldcolor[10]=Green;       
		lightcyan=   oldcolor[3]= Cyan;          
		lightred=    oldcolor[4]= Orange_Red;               
		magenta=     oldcolor[5]=Violet_Red;      
		brown=       oldcolor[6]=Sienna;   
		lightgray=   oldcolor[7]=Light_Steel_Blue;
		darkgray=    oldcolor[8]=Dark_Green;
		lightblue=   oldcolor[9]=Cornflower_Blue;
		lightgreen=  oldcolor[2]=Green_Yellow;
		cyan=        oldcolor[11]=Dark_Turquoise;           
		red=         oldcolor[12]=Salmon;     
		lightmagenta=oldcolor[13]=Pink; 
		yellow=      oldcolor[1]=Yellow;     
                white=       oldcolor[0]=White;      
    }
    
    Number_of_colors=53;
    Color_Table=(int*)calloc( Number_of_colors,sizeof(int));
    
    default_cmap = DefaultColormap(D,S);
    for (i=0;i<Number_of_colors;i++){
        if(!XParseColor(D,default_cmap,color_name[i],&color_def))
                                      printf("\n color absent\n");
        XAllocColor(D,default_cmap,&color_def);
        Color_Table[i]=(int)color_def.pixel;
    }
}    

/********************* show all colors ******************************/
void show_all_colors(void){    
   signed short c,z,thisx,thisy,starty,Number_of_colors;   
   unsigned char pixc;
 
    static char * color_name[]= {
        "White","Black","Yellow", "Gold", "Khaki", "Wheat", 
        "Tan", "Goldenrod", "Orange","Coral","Salmon", 
        "Red", "Orange Red", "Indian Red", "Firebrick", 
        "Brown","Sienna", "Maroon", "Medium Violet Red", 
        "Violet Red","Pink", "Green", "Spring Green", 
        "Medium Spring Green", "Green Yellow",
        "Pale Green","Yellow Green", "Lime Green", 
        "Medium Sea Green","Sea Green", "Forest Green",
        "Olive Drab", "Dark Olive Green", "Dark Green", 
        "Cadet Blue","Medium Aquamarine","Dark Turquoise", 
        "Turquoise", "Aquamarine", "Cyan","Blue", 
        "Medium Blue", "Medium Slate Blue", 
        "Cornflower Blue","Sky Blue", "Light Blue", 
        "Light Steel Blue", "Steel Blue","Dark Slate Blue",
        "Navy", "Navy Blue", "Midnight Blue",
         /******** extra **********/
        "GhostWhite"
    };
    
    Number_of_colors=53;
    XClearWindow(D,W);

    moveto(vgax(5),vgay(50));
    thisx=getx;
    thisy=starty=gety;
    setcolor(white);
    for(c=0;c< Number_of_colors;c++){
        setfillstyle(SOLID_FILL,c);
        bar( thisx,thisy,thisx+30,thisy+15);
        rectangle(thisx,thisy,thisx+30,thisy+15);
        for (z=0;z<=15;z++){
           if (c==oldcolor[z])
             rectangle(thisx-2,thisy-2,thisx+30+2,thisy+15+2);
	}
        pixc=getpixel(thisx+1,thisy+1);
        sprintf(textb[0],"%d ,%d ,%d ,%s",
                 c,Color_Table[c],pixc,color_name[c]);
        moveto(thisx+35,thisy+10);
        outtext(textb[0]);
        moveto(getx-35,gety-10);
        
        if (Maxy-gety<50){
             thisx+=200;
             thisy=starty;
        }
        else thisy+=20;
    }
    setfillstyle(SOLID_FILL,black);
    

}    
/***************** show old colors only ************************/
void show_old_colors(){    
   signed short c,thisx,thisy,starty;    
   unsigned char pixc;
    
    static char * color_name[]= {
        "White","Black","Yellow", "Gold", "Khaki", "Wheat", 
        "Tan", "Goldenrod", "Orange","Coral","Salmon", 
        "Red", "Orange Red", "Indian Red", "Firebrick", 
        "Brown","Sienna", "Maroon", "Medium Violet Red", 
        "Violet Red","Pink", "Green", "Spring Green", 
        "Medium Spring Green", "Green Yellow",
        "Pale Green","Yellow Green", "Lime Green", 
        "Medium Sea Green","Sea Green", "Forest Green",
        "Olive Drab", "Dark Olive Green", "Dark Green", 
        "Cadet Blue","Medium Aquamarine","Dark Turquoise", 
        "Turquoise", "Aquamarine", "Cyan","Blue", 
        "Medium Blue", "Medium Slate Blue", 
        "Cornflower Blue","Sky Blue", "Light Blue", 
        "Light Steel Blue", "Steel Blue","Dark Slate Blue",
        "Navy", "Navy Blue", "Midnight Blue",
         /******** extra **********/
        "GhostWhite"

    };
    /*old color name buf*/ 
    static char * old_color_name[]= {           
        "black","blue","green","lightcyan","lightred","magenta","brown",
        "lightgray","darkgray","lightblue","lightgreen",
        "cyan","red","lightmagenta","yellow","white" 
    };
      
    XClearWindow(D,W);      
    moveto(vgax(5),vgay(50));
    thisx=getx;
    thisy=starty=gety;
    setcolor(white);
    for(c=0;c<=15;c++){
        setfillstyle(SOLID_FILL,oldcolor[c]);
        bar( thisx,thisy,thisx+30,thisy+15);
        rectangle(thisx,thisy,thisx+30,thisy+15);
        pixc=getpixel(thisx+1,thisy+1);
        sprintf(textb[0],"%d: new=%d defno=%d getpix=%d, %s, %s",
                 c,oldcolor[c],Color_Table[oldcolor[c]],pixc,
                   old_color_name[c],color_name[oldcolor[c]]);
        moveto(thisx+35,thisy+10);
        outtext(textb[0]);
        moveto(getx-35,gety-10);
        
        if (Maxy-gety<50){
             thisx+=200;
             thisy=starty;
        }
        else thisy+=20;
    }
    setfillstyle(SOLID_FILL,black);
  
}    

/************************ revise text size ******************************/
/*
void revise_text_size(void){
 signed short xbox,newwidth,newheight;

  start:
       setcolor(white);
       setusercharsize(txd,tx2,tyd,ty2);
       xbox=draw_prompt_box(4);
       sprintf(textb[0],"this is default textsize width-%d x height-%d,",
				      textwidth("z"),textheight("z"));
       outtext(textb[0]);
       moveto(xbox,gety()+textscale);
       myouttext("to alter, enter x factor:");
       input_string(3,1);
       if (is[0]!=13) tx1=(signed short)((float)txd*atof(is));
       if (is[0]=='q') goto end;
       myouttext(" , enter y factor:");
       input_string(3,1);
       if (is[0]!=13) ty1=(signed short)((float)tyd*atof(is));
       if (is[0]=='q') goto end;
       setusercharsize(tx1,tx2,ty1,ty2);
       newwidth=teLxtwidth("z");
       newheight=textheight("z");
       setusercharsize(txd,tx2,tyd,ty2);
       moveto(xbox,gety()+textscale);
       sprintf(textb[0],"new textsize is %d x %d,",newwidth,newheight);
       myouttext(textb[0]);
       moveto(xbox,gety()+textscale);
       myouttext(" revise-r, q-to abandon change, return to accept:");
       input_string(1,0);
       if (is[0]=='r') goto start;
       if (is[0]==13) setusercharsize(tx1,tx2,ty1,ty2);
  end:
       cleardevice();
}
*/





